﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;

namespace AgeSpanTest
{
    
    /// <summary>
    ///This is a test class for AgeSpanTest and is intended
    ///to contain all AgeSpanTest Unit Tests
    ///</summary>
    [TestClass()]
    public class AgeSpanTest
    {


        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region .Simple tests no leap year

        /// <summary>
        /// Testing for sample date plus one day, no leap year
        ///</summary>
        [TestMethod()]
        public void test_for_sample_plus_one_day_no_leap_year()
        {
            DateTime birthDate = new DateTime(1990, 10, 3);
            DateTime asOfDate = birthDate.AddDays(1);

            AgeSpan target = new AgeSpan(birthDate, asOfDate);

            string expected = "years:0|months:0|days:1|total_months:0|total_days:1";
            string actual;
            actual = target.ToString();
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        /// Testing for sample date plus one month, no leap year
        ///</summary>
        [TestMethod()]
        public void test_for_sample_plus_one_month_no_leap_year()
        {
            DateTime birthDate = new DateTime(1990, 10, 3);
            DateTime asOfDate = birthDate.AddMonths(1);

            AgeSpan target = new AgeSpan(birthDate, asOfDate);

            string expected = "years:0|months:1|days:0|total_months:1|total_days:31";
            string actual;
            actual = target.ToString();
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        /// Testing for sample date plus one year, no leap year
        ///</summary>
        [TestMethod()]
        public void test_for_sample_plus_one_year_no_leap_year()
        {
            DateTime birthDate = new DateTime(1990, 10, 3);
            DateTime asOfDate = birthDate.AddYears(1);

            AgeSpan target = new AgeSpan(birthDate, asOfDate);

            string expected = "years:1|months:0|days:0|total_months:12|total_days:365";
            string actual;
            actual = target.ToString();
            Assert.AreEqual(expected, actual);
        }

        #endregion

        #region .Simple tests crossing leap year boundry

        /// <summary>
        /// Testing for sample date plus one month, includes a leap year
        ///</summary>
        [TestMethod()]
        public void test_for_sample_plus_one_month_include_leap_year()
        {
            DateTime birthDate = new DateTime(2004, 2, 3);
            DateTime asOfDate = birthDate.AddMonths(1);

            AgeSpan target = new AgeSpan(birthDate, asOfDate);

            string expected = "years:0|months:1|days:0|total_months:1|total_days:29";
            string actual;
            actual = target.ToString();
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        /// Testing for sample date plus one year, includes a leap year
        ///</summary>
        [TestMethod()]
        public void test_for_sample_plus_one_year_include_leap_year()
        {
            DateTime birthDate = new DateTime(2004, 2, 3);
            DateTime asOfDate = birthDate.AddYears(1);

            AgeSpan target = new AgeSpan(birthDate, asOfDate);

            string expected = "years:1|months:0|days:0|total_months:12|total_days:366";
            string actual;
            actual = target.ToString();
            Assert.AreEqual(expected, actual);
        }

        #endregion

        #region .Two year tests multiple month/day crossovers

        /// <summary>
        /// Testing for sample date plus 2 years, month same as sample month, day same as sample day
        ///</summary>
        [TestMethod()]
        public void test_for_sample_plus_2_years_month_same_day_same()
        {
            DateTime birthDate = new DateTime(2005, 5, 6);
            DateTime asOfDate = new DateTime(2007, 5, 6);

            AgeSpan target = new AgeSpan(birthDate, asOfDate);

            string expected = "years:2|months:0|days:0|total_months:24|total_days:730";
            string actual;
            actual = target.ToString();
            Assert.AreEqual(expected, actual);

        }

        /// <summary>
        /// Testing for sample date plus 2 years, 1 month less than sample month, 1 day less than sample day
        ///</summary>
        [TestMethod()]
        public void test_for_sample_plus_2_years_1_month_less_1_day_less()
        {
            DateTime birthDate = new DateTime(2005, 5, 6);
            DateTime asOfDate = new DateTime(2007, 4, 5);

            AgeSpan target = new AgeSpan(birthDate, asOfDate);

            string expected = "years:1|months:10|days:30|total_months:22|total_days:699";
            string actual;
            actual = target.ToString();
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        /// Testing for sample date plus 2 years, 2 month less than sample month, 1 day less than sample day
        ///</summary>
        [TestMethod()]
        public void test_for_sample_plus_2_years_2_month_less_1_day_less()
        {
            DateTime birthDate = new DateTime(2005, 5, 6);
            DateTime asOfDate = new DateTime(2007, 3, 5);

            AgeSpan target = new AgeSpan(birthDate, asOfDate);

            string expected = "years:1|months:9|days:27|total_months:21|total_days:668";
            string actual;
            actual = target.ToString();
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        /// Testing for sample date plus 2 years, 1 month less than sample month, 1 day more than sample day
        ///</summary>
        [TestMethod()]
        public void test_for_sample_plus_2_years_1_month_less_1_day_more()
        {
            DateTime birthDate = new DateTime(2005, 5, 6);
            DateTime asOfDate = new DateTime(2007, 4, 7);

            AgeSpan target = new AgeSpan(birthDate, asOfDate);

            string expected = "years:1|months:11|days:1|total_months:23|total_days:701";
            string actual;
            actual = target.ToString();
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        /// Testing for sample date plus 2 years, 1 month more than sample month, 1 day more than sample day
        ///</summary>
        [TestMethod()]
        public void test_for_sample_plus_2_years_1_month_more_1_day_more()
        {
            DateTime birthDate = new DateTime(2005, 5, 6);
            DateTime asOfDate = new DateTime(2007, 6, 7);

            AgeSpan target = new AgeSpan(birthDate, asOfDate);

            string expected = "years:2|months:1|days:1|total_months:25|total_days:762";
            string actual;
            actual = target.ToString();
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        /// Testing for sample date plus 2 years, 1 month more than sample month, 1 day less than sample day
        ///</summary>
        [TestMethod()]
        public void test_for_sample_plus_2_years_1_month_more_1_day_less()
        {
            DateTime birthDate = new DateTime(2005, 5, 6);
            DateTime asOfDate = new DateTime(2007, 6, 5);

            AgeSpan target = new AgeSpan(birthDate, asOfDate);

            string expected = "years:2|months:0|days:30|total_months:24|total_days:760";
            string actual;
            actual = target.ToString();
            Assert.AreEqual(expected, actual);
        }

        #endregion

        #region .Same year tests with month/day crossovers

        /// <summary>
        /// Testing for sample date in same year, month same as sample month, day same as sample day
        ///</summary>
        [TestMethod()]
        public void test_for_sample_in_same_year_month_same_day_same()
        {
            DateTime birthDate = new DateTime(2005, 5, 6);
            DateTime asOfDate = new DateTime(2005, 5, 6);

            AgeSpan target = new AgeSpan(birthDate, asOfDate);

            string expected = "years:0|months:0|days:0|total_months:0|total_days:0";
            string actual;
            actual = target.ToString();
            Assert.AreEqual(expected, actual);

        }

        /// <summary>
        /// Testing for sample date in same year, 3 month more than sample month, 1 day less than sample day
        ///</summary>
        [TestMethod()]
        public void test_for_sample_same_year_3_month_more_1_day_less()
        {
            DateTime birthDate = new DateTime(2007, 5, 6);
            DateTime asOfDate = new DateTime(2007, 8, 5);

            AgeSpan target = new AgeSpan(birthDate, asOfDate);

            string expected = "years:0|months:2|days:30|total_months:2|total_days:91";
            string actual;
            actual = target.ToString();
            Assert.AreEqual(expected, actual);
        }

        /// <summary>
        /// Testing for sample date in same year, 3 month more than sample month, 1 day more than sample day
        ///</summary>
        [TestMethod()]
        public void test_for_sample_same_year_3_month_more_1_day_more()
        {
            DateTime birthDate = new DateTime(2007, 5, 6);
            DateTime asOfDate = new DateTime(2007, 8, 7);

            AgeSpan target = new AgeSpan(birthDate, asOfDate);

            string expected = "years:0|months:3|days:1|total_months:3|total_days:93";
            string actual;
            actual = target.ToString();
            Assert.AreEqual(expected, actual);
        }

        #endregion
    }
}